! start of self-replicating code

module selfrepmodule

   implicit none

   private

   integer, parameter              :: shortint = selected_int_kind(3)
   character(*), parameter         :: MARKER   = '! '//'This is the MARKER'

   !                                  ! need to make sure MARKER is not found here,
   !                                  ! that is the reason for //

   integer, parameter              :: markerfound    = -1
   integer, parameter              :: endoffilefound = -2

   ! Following declaration of A and B are to assure that in s1.f90
   ! A and B are defined for subroutine selfreplicate.
   ! In the self replicating version of the program, the definitions
   ! of A and B are created in the subroutine itself, thus shadowing
   ! definition below

   integer(shortint), dimension(0) :: A,B

   public                          :: makeselfrep,selfreplicate

contains

   subroutine PRINT1(x)

      implicit none
      integer(shortint), intent(in) :: x(:)
      integer :: i

      do i=1,size(x)
         write (*,'(a)',advance='no') char(x(i))
      enddo

   end subroutine PRINT1


   subroutine PRINT2(n,x)

      implicit none
      character(*),      intent(in) :: n
      integer(shortint), intent(in) :: x(:)
      integer                       :: i,j,jmax,k
      integer, parameter            :: l=20     ! max number of numbers per line
      integer, parameter            :: ll=20    ! max number of continuation lines
      integer, parameter            :: lll = l*ll

      do i=1,size(x),lll
         jmax = min(size(x),i+lll-1)
         write(*,'(a,a,a,i0,a,i0,a)') 'data ',n,'(',i,':',jmax,')/&'
         k = 0
         do j=i,jmax
            write(*,'(i0)',advance='no') x(j)
            if (j .ne. jmax) write(*,'(a)',advance='no') ','
            k = k+1
            if ( k .ge. l) then
               write(*,'(a)') '&'
               k = 0
            endif
         enddo
         write(*,'(a)') '/'
      enddo

      if(size(A) .eq. -1) A = B  ! to avoid warning about unused a and b

   end subroutine PRINT2


   integer function fmarker()

      ! reads from unit 11
      ! returns:
      !    markerfound:    MARKER found
      !    endoffilefound: end of file found
      !    other:          ichar() of character read

      implicit none
      integer i,p
      character c

      inquire(11,pos=p)

      do i=1,len(MARKER)
         read(11,end=100) c
         if (c .ne. MARKER(i:i)) goto 100
      enddo

      fmarker = markerfound

      return

100   continue

      read(11,pos=p,end=200) c
      fmarker = ichar(c)
      return

200   continue

      fmarker = endoffilefound
      return

   end function fmarker


   subroutine printselfrepprog(A,B,mode)

      implicit none
      integer(shortint), intent(in) :: A(:),B(:)
      integer,           intent(in) :: mode

      call PRINT1(A)

      if (mode .eq. 0) then

         ! This will be written directly after the MARKER,
         ! so a new line is necessary

         write(*,*)
         write(*,'(a,i0,a)') 'integer(shortint) :: A(',size(A),')'
         write(*,'(a,i0,a)') 'integer(shortint) :: B(',size(B),')'

         call PRINT2('A',A)
         call PRINT2('B',B)
      endif

      call PRINT1(B)

   endsubroutine printselfrepprog


   subroutine selfreplicate(mode)

      implicit none
      integer, intent(in) :: mode

      ! in THIS program, selfreplicate is not used,
      ! the definitions for A and B are in the module above
      ! in the generated program, definitions for A and B will
      ! appear just after the MARKER

      ! mode  = 0: replicate the whole program
      ! mode != 0: replicate original program

      ! This is the MARKER

      if (size(A) .eq. 0) then
         write(*,'(a)') 'This is not a self replicating program'
         stop
      endif

      call printselfrepprog(A,B,mode)

   end subroutine selfreplicate


   subroutine makeselfrep(f)

      implicit none
      character(*), intent(in)       :: f  ! file to read
      integer                        :: fm,j,i
      character                      :: c
      integer(shortint), allocatable :: A(:)
      integer(shortint), allocatable :: B(:)
      integer sizea, sizeb

      ! read file until and including MARKER.
      ! stores this as ichar() values in A
      ! number of characters stored in sizea
      ! reads from end of MARKER to end of file
      ! stores this as ichar() values in B
      ! number of characters stored in sizeb

      ! determine number of characters needed for A and B
      ! then allocate A and B
      !      and read A and B

      open(11,file=f,access='stream')

      sizea = 0
      do
         fm = fmarker()
         select case (fm)
          case(markerfound)
            do j=1,len(MARKER)
               sizea = sizea + 1
            enddo
            exit
          case(endoffilefound)
            exit
          case default
            sizea = sizea + 1
         endselect
      enddo

      sizeb = 0
      do
         read (11,end=150) c
         sizeb = sizeb + 1
      enddo
150   continue

      rewind 11

      allocate(A(sizea))
      allocate(B(sizeb))

      do i=1,sizea
         read(11)c
         A(i) = int(ichar(c),shortint)
      enddo

      do i=1,sizeb
         read(11) c
         B(i) = int(ichar(c),shortint)
      enddo

      close(11)

      call printselfrepprog(A,B,0)

   end subroutine makeselfrep

end module selfrepmodule

! end of self-replicating code

program hello

   ! This is the main program. Depending on the value of the
   ! environment variable HELLO_TODO, it will
   ! reproduce the sef-reproducing source, reproduce THIS source,
   ! or print 'Hello world'

   ! HELLO_TODO=
   ! "" or absent: run in normal mode (print "Hello world")
   ! "replicate" : print self reproducing source to stdout
   ! "original" : print THIS source to stdout
   ! Otherwize the value is taken as a file name. This file
   ! will be read and converted to self-replicating program.
   !    Normally, the name of THIS file will be given.

   use selfrepmodule

   implicit none

   character(1000) :: todo

   call get_environment_variable("HELLO_TODO",todo)

   select case(todo)

    case("replicate")
      call selfreplicate(0)

    case("original")
      call selfreplicate(1)

    case(" ")
      print *,"Hello world"

    case default
      call makeselfrep(todo)   ! use todo as file name

   end select

end program hello
