! start of self-replicating code

module selfrepmodule

   implicit none

   private

   integer, parameter              :: shortint = selected_int_kind(3)
   character(*), parameter         :: MARKER   = '! '//'This is the MARKER'

   !                                  ! need to make sure MARKER is not found here,
   !                                  ! that is the reason for //

   integer, parameter              :: markerfound    = -1
   integer, parameter              :: endoffilefound = -2

   ! Following declaration of A and B are to assure that in s1.f90
   ! A and B are defined for subroutine selfreplicate.
   ! In the self replicating version of the program, the definitions
   ! of A and B are created in the subroutine itself, thus shadowing
   ! definition below

   integer(shortint), dimension(0) :: A,B

   public                          :: makeselfrep,selfreplicate

contains

   subroutine PRINT1(x)

      implicit none
      integer(shortint), intent(in) :: x(:)
      integer :: i

      do i=1,size(x)
         write (*,'(a)',advance='no') char(x(i))
      enddo

   end subroutine PRINT1


   subroutine PRINT2(n,x)

      implicit none
      character(*),      intent(in) :: n
      integer(shortint), intent(in) :: x(:)
      integer                       :: i,j,jmax,k
      integer, parameter            :: l=20     ! max number of numbers per line
      integer, parameter            :: ll=20    ! max number of continuation lines
      integer, parameter            :: lll = l*ll

      do i=1,size(x),lll
         jmax = min(size(x),i+lll-1)
         write(*,'(a,a,a,i0,a,i0,a)') 'data ',n,'(',i,':',jmax,')/&'
         k = 0
         do j=i,jmax
            write(*,'(i0)',advance='no') x(j)
            if (j .ne. jmax) write(*,'(a)',advance='no') ','
            k = k+1
            if ( k .ge. l) then
               write(*,'(a)') '&'
               k = 0
            endif
         enddo
         write(*,'(a)') '/'
      enddo

      if(size(A) .eq. -1) A = B  ! to avoid warning about unused a and b

   end subroutine PRINT2


   integer function fmarker()

      ! reads from unit 11
      ! returns:
      !    markerfound:    MARKER found
      !    endoffilefound: end of file found
      !    other:          ichar() of character read

      implicit none
      integer i,p
      character c

      inquire(11,pos=p)

      do i=1,len(MARKER)
         read(11,end=100) c
         if (c .ne. MARKER(i:i)) goto 100
      enddo

      fmarker = markerfound

      return

100   continue

      read(11,pos=p,end=200) c
      fmarker = ichar(c)
      return

200   continue

      fmarker = endoffilefound
      return

   end function fmarker


   subroutine printselfrepprog(A,B,mode)

      implicit none
      integer(shortint), intent(in) :: A(:),B(:)
      integer,           intent(in) :: mode

      call PRINT1(A)

      if (mode .eq. 0) then

         ! This will be written directly after the MARKER,
         ! so a new line is necessary

         write(*,*)
         write(*,'(a,i0,a)') 'integer(shortint) :: A(',size(A),')'
         write(*,'(a,i0,a)') 'integer(shortint) :: B(',size(B),')'

         call PRINT2('A',A)
         call PRINT2('B',B)
      endif

      call PRINT1(B)

   endsubroutine printselfrepprog


   subroutine selfreplicate(mode)

      implicit none
      integer, intent(in) :: mode

      ! in THIS program, selfreplicate is not used,
      ! the definitions for A and B are in the module above
      ! in the generated program, definitions for A and B will
      ! appear just after the MARKER

      ! mode  = 0: replicate the whole program
      ! mode != 0: replicate original program

      ! This is the MARKER
integer(shortint) :: A(3704)
integer(shortint) :: B(2746)
data A(1:400)/&
33,32,115,116,97,114,116,32,111,102,32,115,101,108,102,45,114,101,112,108,&
105,99,97,116,105,110,103,32,99,111,100,101,10,10,109,111,100,117,108,101,&
32,115,101,108,102,114,101,112,109,111,100,117,108,101,10,10,32,32,32,105,&
109,112,108,105,99,105,116,32,110,111,110,101,10,10,32,32,32,112,114,105,&
118,97,116,101,10,10,32,32,32,105,110,116,101,103,101,114,44,32,112,97,&
114,97,109,101,116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,58,58,32,115,104,111,114,116,105,110,116,32,61,32,115,101,108,101,99,&
116,101,100,95,105,110,116,95,107,105,110,100,40,51,41,10,32,32,32,99,&
104,97,114,97,99,116,101,114,40,42,41,44,32,112,97,114,97,109,101,116,&
101,114,32,32,32,32,32,32,32,32,32,58,58,32,77,65,82,75,69,82,&
32,32,32,61,32,39,33,32,39,47,47,39,84,104,105,115,32,105,115,32,&
116,104,101,32,77,65,82,75,69,82,39,10,10,32,32,32,33,32,32,32,&
32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,32,32,32,32,32,32,32,32,32,33,32,110,101,101,100,32,116,111,&
32,109,97,107,101,32,115,117,114,101,32,77,65,82,75,69,82,32,105,115,&
32,110,111,116,32,102,111,117,110,100,32,104,101,114,101,44,10,32,32,32,&
33,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,33,32,116,104,97,&
116,32,105,115,32,116,104,101,32,114,101,97,115,111,110,32,102,111,114,32,&
47,47,10,10,32,32,32,105,110,116,101,103,101,114,44,32,112,97,114,97&
/
data A(401:800)/&
109,101,116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,32,32,58,&
58,32,109,97,114,107,101,114,102,111,117,110,100,32,32,32,32,61,32,45,&
49,10,32,32,32,105,110,116,101,103,101,114,44,32,112,97,114,97,109,101,&
116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,32,32,58,58,32,&
101,110,100,111,102,102,105,108,101,102,111,117,110,100,32,61,32,45,50,10,&
10,32,32,32,33,32,70,111,108,108,111,119,105,110,103,32,100,101,99,108,&
97,114,97,116,105,111,110,32,111,102,32,65,32,97,110,100,32,66,32,97,&
114,101,32,116,111,32,97,115,115,117,114,101,32,116,104,97,116,32,105,110,&
32,115,49,46,102,57,48,10,32,32,32,33,32,65,32,97,110,100,32,66,&
32,97,114,101,32,100,101,102,105,110,101,100,32,102,111,114,32,115,117,98,&
114,111,117,116,105,110,101,32,115,101,108,102,114,101,112,108,105,99,97,116,&
101,46,10,32,32,32,33,32,73,110,32,116,104,101,32,115,101,108,102,32,&
114,101,112,108,105,99,97,116,105,110,103,32,118,101,114,115,105,111,110,32,&
111,102,32,116,104,101,32,112,114,111,103,114,97,109,44,32,116,104,101,32,&
100,101,102,105,110,105,116,105,111,110,115,10,32,32,32,33,32,111,102,32,&
65,32,97,110,100,32,66,32,97,114,101,32,99,114,101,97,116,101,100,32,&
105,110,32,116,104,101,32,115,117,98,114,111,117,116,105,110,101,32,105,116,&
115,101,108,102,44,32,116,104,117,115,32,115,104,97,100,111,119,105,110,103,&
10,32,32,32,33,32,100,101,102,105,110,105,116,105,111,110,32,98,101,108,&
111,119,10,10,32,32,32,105,110,116,101,103,101,114,40,115,104,111,114,116&
/
data A(801:1200)/&
105,110,116,41,44,32,100,105,109,101,110,115,105,111,110,40,48,41,32,58,&
58,32,65,44,66,10,10,32,32,32,112,117,98,108,105,99,32,32,32,32,&
32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,58,58,32,109,97,107,101,115,101,108,102,114,101,112,44,115,101,108,&
102,114,101,112,108,105,99,97,116,101,10,10,99,111,110,116,97,105,110,115,&
10,10,32,32,32,115,117,98,114,111,117,116,105,110,101,32,80,82,73,78,&
84,49,40,120,41,10,10,32,32,32,32,32,32,105,109,112,108,105,99,105,&
116,32,110,111,110,101,10,32,32,32,32,32,32,105,110,116,101,103,101,114,&
40,115,104,111,114,116,105,110,116,41,44,32,105,110,116,101,110,116,40,105,&
110,41,32,58,58,32,120,40,58,41,10,32,32,32,32,32,32,105,110,116,&
101,103,101,114,32,58,58,32,105,10,10,32,32,32,32,32,32,100,111,32,&
105,61,49,44,115,105,122,101,40,120,41,10,32,32,32,32,32,32,32,32,&
32,119,114,105,116,101,32,40,42,44,39,40,97,41,39,44,97,100,118,97,&
110,99,101,61,39,110,111,39,41,32,99,104,97,114,40,120,40,105,41,41,&
10,32,32,32,32,32,32,101,110,100,100,111,10,10,32,32,32,101,110,100,&
32,115,117,98,114,111,117,116,105,110,101,32,80,82,73,78,84,49,10,10,&
10,32,32,32,115,117,98,114,111,117,116,105,110,101,32,80,82,73,78,84,&
50,40,110,44,120,41,10,10,32,32,32,32,32,32,105,109,112,108,105,99,&
105,116,32,110,111,110,101,10,32,32,32,32,32,32,99,104,97,114,97,99,&
116,101,114,40,42,41,44,32,32,32,32,32,32,105,110,116,101,110,116,40&
/
data A(1201:1600)/&
105,110,41,32,58,58,32,110,10,32,32,32,32,32,32,105,110,116,101,103,&
101,114,40,115,104,111,114,116,105,110,116,41,44,32,105,110,116,101,110,116,&
40,105,110,41,32,58,58,32,120,40,58,41,10,32,32,32,32,32,32,105,&
110,116,101,103,101,114,32,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,32,32,32,32,32,32,32,58,58,32,105,44,106,44,106,109,97,120,&
44,107,10,32,32,32,32,32,32,105,110,116,101,103,101,114,44,32,112,97,&
114,97,109,101,116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,58,&
58,32,108,61,50,48,32,32,32,32,32,33,32,109,97,120,32,110,117,109,&
98,101,114,32,111,102,32,110,117,109,98,101,114,115,32,112,101,114,32,108,&
105,110,101,10,32,32,32,32,32,32,105,110,116,101,103,101,114,44,32,112,&
97,114,97,109,101,116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,&
58,58,32,108,108,61,50,48,32,32,32,32,33,32,109,97,120,32,110,117,&
109,98,101,114,32,111,102,32,99,111,110,116,105,110,117,97,116,105,111,110,&
32,108,105,110,101,115,10,32,32,32,32,32,32,105,110,116,101,103,101,114,&
44,32,112,97,114,97,109,101,116,101,114,32,32,32,32,32,32,32,32,32,&
32,32,32,58,58,32,108,108,108,32,61,32,108,42,108,108,10,10,32,32,&
32,32,32,32,100,111,32,105,61,49,44,115,105,122,101,40,120,41,44,108,&
108,108,10,32,32,32,32,32,32,32,32,32,106,109,97,120,32,61,32,109,&
105,110,40,115,105,122,101,40,120,41,44,105,43,108,108,108,45,49,41,10,&
32,32,32,32,32,32,32,32,32,119,114,105,116,101,40,42,44,39,40,97&
/
data A(1601:2000)/&
44,97,44,97,44,105,48,44,97,44,105,48,44,97,41,39,41,32,39,100,&
97,116,97,32,39,44,110,44,39,40,39,44,105,44,39,58,39,44,106,109,&
97,120,44,39,41,47,38,39,10,32,32,32,32,32,32,32,32,32,107,32,&
61,32,48,10,32,32,32,32,32,32,32,32,32,100,111,32,106,61,105,44,&
106,109,97,120,10,32,32,32,32,32,32,32,32,32,32,32,32,119,114,105,&
116,101,40,42,44,39,40,105,48,41,39,44,97,100,118,97,110,99,101,61,&
39,110,111,39,41,32,120,40,106,41,10,32,32,32,32,32,32,32,32,32,&
32,32,32,105,102,32,40,106,32,46,110,101,46,32,106,109,97,120,41,32,&
119,114,105,116,101,40,42,44,39,40,97,41,39,44,97,100,118,97,110,99,&
101,61,39,110,111,39,41,32,39,44,39,10,32,32,32,32,32,32,32,32,&
32,32,32,32,107,32,61,32,107,43,49,10,32,32,32,32,32,32,32,32,&
32,32,32,32,105,102,32,40,32,107,32,46,103,101,46,32,108,41,32,116,&
104,101,110,10,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,119,&
114,105,116,101,40,42,44,39,40,97,41,39,41,32,39,38,39,10,32,32,&
32,32,32,32,32,32,32,32,32,32,32,32,32,107,32,61,32,48,10,32,&
32,32,32,32,32,32,32,32,32,32,32,101,110,100,105,102,10,32,32,32,&
32,32,32,32,32,32,101,110,100,100,111,10,32,32,32,32,32,32,32,32,&
32,119,114,105,116,101,40,42,44,39,40,97,41,39,41,32,39,47,39,10,&
32,32,32,32,32,32,101,110,100,100,111,10,10,32,32,32,32,32,32,105,&
102,40,115,105,122,101,40,65,41,32,46,101,113,46,32,45,49,41,32,65&
/
data A(2001:2400)/&
32,61,32,66,32,32,33,32,116,111,32,97,118,111,105,100,32,119,97,114,&
110,105,110,103,32,97,98,111,117,116,32,117,110,117,115,101,100,32,97,32,&
97,110,100,32,98,10,10,32,32,32,101,110,100,32,115,117,98,114,111,117,&
116,105,110,101,32,80,82,73,78,84,50,10,10,10,32,32,32,105,110,116,&
101,103,101,114,32,102,117,110,99,116,105,111,110,32,102,109,97,114,107,101,&
114,40,41,10,10,32,32,32,32,32,32,33,32,114,101,97,100,115,32,102,&
114,111,109,32,117,110,105,116,32,49,49,10,32,32,32,32,32,32,33,32,&
114,101,116,117,114,110,115,58,10,32,32,32,32,32,32,33,32,32,32,32,&
109,97,114,107,101,114,102,111,117,110,100,58,32,32,32,32,77,65,82,75,&
69,82,32,102,111,117,110,100,10,32,32,32,32,32,32,33,32,32,32,32,&
101,110,100,111,102,102,105,108,101,102,111,117,110,100,58,32,101,110,100,32,&
111,102,32,102,105,108,101,32,102,111,117,110,100,10,32,32,32,32,32,32,&
33,32,32,32,32,111,116,104,101,114,58,32,32,32,32,32,32,32,32,32,&
32,105,99,104,97,114,40,41,32,111,102,32,99,104,97,114,97,99,116,101,&
114,32,114,101,97,100,10,10,32,32,32,32,32,32,105,109,112,108,105,99,&
105,116,32,110,111,110,101,10,32,32,32,32,32,32,105,110,116,101,103,101,&
114,32,105,44,112,10,32,32,32,32,32,32,99,104,97,114,97,99,116,101,&
114,32,99,10,10,32,32,32,32,32,32,105,110,113,117,105,114,101,40,49,&
49,44,112,111,115,61,112,41,10,10,32,32,32,32,32,32,100,111,32,105,&
61,49,44,108,101,110,40,77,65,82,75,69,82,41,10,32,32,32,32,32&
/
data A(2401:2800)/&
32,32,32,32,114,101,97,100,40,49,49,44,101,110,100,61,49,48,48,41,&
32,99,10,32,32,32,32,32,32,32,32,32,105,102,32,40,99,32,46,110,&
101,46,32,77,65,82,75,69,82,40,105,58,105,41,41,32,103,111,116,111,&
32,49,48,48,10,32,32,32,32,32,32,101,110,100,100,111,10,10,32,32,&
32,32,32,32,102,109,97,114,107,101,114,32,61,32,109,97,114,107,101,114,&
102,111,117,110,100,10,10,32,32,32,32,32,32,114,101,116,117,114,110,10,&
10,49,48,48,32,32,32,99,111,110,116,105,110,117,101,10,10,32,32,32,&
32,32,32,114,101,97,100,40,49,49,44,112,111,115,61,112,44,101,110,100,&
61,50,48,48,41,32,99,10,32,32,32,32,32,32,102,109,97,114,107,101,&
114,32,61,32,105,99,104,97,114,40,99,41,10,32,32,32,32,32,32,114,&
101,116,117,114,110,10,10,50,48,48,32,32,32,99,111,110,116,105,110,117,&
101,10,10,32,32,32,32,32,32,102,109,97,114,107,101,114,32,61,32,101,&
110,100,111,102,102,105,108,101,102,111,117,110,100,10,32,32,32,32,32,32,&
114,101,116,117,114,110,10,10,32,32,32,101,110,100,32,102,117,110,99,116,&
105,111,110,32,102,109,97,114,107,101,114,10,10,10,32,32,32,115,117,98,&
114,111,117,116,105,110,101,32,112,114,105,110,116,115,101,108,102,114,101,112,&
112,114,111,103,40,65,44,66,44,109,111,100,101,41,10,10,32,32,32,32,&
32,32,105,109,112,108,105,99,105,116,32,110,111,110,101,10,32,32,32,32,&
32,32,105,110,116,101,103,101,114,40,115,104,111,114,116,105,110,116,41,44,&
32,105,110,116,101,110,116,40,105,110,41,32,58,58,32,65,40,58,41,44&
/
data A(2801:3200)/&
66,40,58,41,10,32,32,32,32,32,32,105,110,116,101,103,101,114,44,32,&
32,32,32,32,32,32,32,32,32,32,105,110,116,101,110,116,40,105,110,41,&
32,58,58,32,109,111,100,101,10,10,32,32,32,32,32,32,99,97,108,108,&
32,80,82,73,78,84,49,40,65,41,10,10,32,32,32,32,32,32,105,102,&
32,40,109,111,100,101,32,46,101,113,46,32,48,41,32,116,104,101,110,10,&
10,32,32,32,32,32,32,32,32,32,33,32,84,104,105,115,32,119,105,108,&
108,32,98,101,32,119,114,105,116,116,101,110,32,100,105,114,101,99,116,108,&
121,32,97,102,116,101,114,32,116,104,101,32,77,65,82,75,69,82,44,10,&
32,32,32,32,32,32,32,32,32,33,32,115,111,32,97,32,110,101,119,32,&
108,105,110,101,32,105,115,32,110,101,99,101,115,115,97,114,121,10,10,32,&
32,32,32,32,32,32,32,32,119,114,105,116,101,40,42,44,42,41,10,32,&
32,32,32,32,32,32,32,32,119,114,105,116,101,40,42,44,39,40,97,44,&
105,48,44,97,41,39,41,32,39,105,110,116,101,103,101,114,40,115,104,111,&
114,116,105,110,116,41,32,58,58,32,65,40,39,44,115,105,122,101,40,65,&
41,44,39,41,39,10,32,32,32,32,32,32,32,32,32,119,114,105,116,101,&
40,42,44,39,40,97,44,105,48,44,97,41,39,41,32,39,105,110,116,101,&
103,101,114,40,115,104,111,114,116,105,110,116,41,32,58,58,32,66,40,39,&
44,115,105,122,101,40,66,41,44,39,41,39,10,10,32,32,32,32,32,32,&
32,32,32,99,97,108,108,32,80,82,73,78,84,50,40,39,65,39,44,65,&
41,10,32,32,32,32,32,32,32,32,32,99,97,108,108,32,80,82,73,78&
/
data A(3201:3600)/&
84,50,40,39,66,39,44,66,41,10,32,32,32,32,32,32,101,110,100,105,&
102,10,10,32,32,32,32,32,32,99,97,108,108,32,80,82,73,78,84,49,&
40,66,41,10,10,32,32,32,101,110,100,115,117,98,114,111,117,116,105,110,&
101,32,112,114,105,110,116,115,101,108,102,114,101,112,112,114,111,103,10,10,&
10,32,32,32,115,117,98,114,111,117,116,105,110,101,32,115,101,108,102,114,&
101,112,108,105,99,97,116,101,40,109,111,100,101,41,10,10,32,32,32,32,&
32,32,105,109,112,108,105,99,105,116,32,110,111,110,101,10,32,32,32,32,&
32,32,105,110,116,101,103,101,114,44,32,105,110,116,101,110,116,40,105,110,&
41,32,58,58,32,109,111,100,101,10,10,32,32,32,32,32,32,33,32,105,&
110,32,84,72,73,83,32,112,114,111,103,114,97,109,44,32,115,101,108,102,&
114,101,112,108,105,99,97,116,101,32,105,115,32,110,111,116,32,117,115,101,&
100,44,10,32,32,32,32,32,32,33,32,116,104,101,32,100,101,102,105,110,&
105,116,105,111,110,115,32,102,111,114,32,65,32,97,110,100,32,66,32,97,&
114,101,32,105,110,32,116,104,101,32,109,111,100,117,108,101,32,97,98,111,&
118,101,10,32,32,32,32,32,32,33,32,105,110,32,116,104,101,32,103,101,&
110,101,114,97,116,101,100,32,112,114,111,103,114,97,109,44,32,100,101,102,&
105,110,105,116,105,111,110,115,32,102,111,114,32,65,32,97,110,100,32,66,&
32,119,105,108,108,10,32,32,32,32,32,32,33,32,97,112,112,101,97,114,&
32,106,117,115,116,32,97,102,116,101,114,32,116,104,101,32,77,65,82,75,&
69,82,10,10,32,32,32,32,32,32,33,32,109,111,100,101,32,32,61,32&
/
data A(3601:3704)/&
48,58,32,114,101,112,108,105,99,97,116,101,32,116,104,101,32,119,104,111,&
108,101,32,112,114,111,103,114,97,109,10,32,32,32,32,32,32,33,32,109,&
111,100,101,32,33,61,32,48,58,32,114,101,112,108,105,99,97,116,101,32,&
111,114,105,103,105,110,97,108,32,112,114,111,103,114,97,109,10,10,32,32,&
32,32,32,32,33,32,84,104,105,115,32,105,115,32,116,104,101,32,77,65,&
82,75,69,82/
data B(1:400)/&
10,10,32,32,32,32,32,32,105,102,32,40,115,105,122,101,40,65,41,32,&
46,101,113,46,32,48,41,32,116,104,101,110,10,32,32,32,32,32,32,32,&
32,32,119,114,105,116,101,40,42,44,39,40,97,41,39,41,32,39,84,104,&
105,115,32,105,115,32,110,111,116,32,97,32,115,101,108,102,32,114,101,112,&
108,105,99,97,116,105,110,103,32,112,114,111,103,114,97,109,39,10,32,32,&
32,32,32,32,32,32,32,115,116,111,112,10,32,32,32,32,32,32,101,110,&
100,105,102,10,10,32,32,32,32,32,32,99,97,108,108,32,112,114,105,110,&
116,115,101,108,102,114,101,112,112,114,111,103,40,65,44,66,44,109,111,100,&
101,41,10,10,32,32,32,101,110,100,32,115,117,98,114,111,117,116,105,110,&
101,32,115,101,108,102,114,101,112,108,105,99,97,116,101,10,10,10,32,32,&
32,115,117,98,114,111,117,116,105,110,101,32,109,97,107,101,115,101,108,102,&
114,101,112,40,102,41,10,10,32,32,32,32,32,32,105,109,112,108,105,99,&
105,116,32,110,111,110,101,10,32,32,32,32,32,32,99,104,97,114,97,99,&
116,101,114,40,42,41,44,32,105,110,116,101,110,116,40,105,110,41,32,32,&
32,32,32,32,32,58,58,32,102,32,32,33,32,102,105,108,101,32,116,111,&
32,114,101,97,100,10,32,32,32,32,32,32,105,110,116,101,103,101,114,32,&
32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,32,58,58,32,102,109,44,106,44,105,10,32,32,32,32,32,32,99,&
104,97,114,97,99,116,101,114,32,32,32,32,32,32,32,32,32,32,32,32,&
32,32,32,32,32,32,32,32,32,32,58,58,32,99,10,32,32,32,32,32&
/
data B(401:800)/&
32,105,110,116,101,103,101,114,40,115,104,111,114,116,105,110,116,41,44,32,&
97,108,108,111,99,97,116,97,98,108,101,32,58,58,32,65,40,58,41,10,&
32,32,32,32,32,32,105,110,116,101,103,101,114,40,115,104,111,114,116,105,&
110,116,41,44,32,97,108,108,111,99,97,116,97,98,108,101,32,58,58,32,&
66,40,58,41,10,32,32,32,32,32,32,105,110,116,101,103,101,114,32,115,&
105,122,101,97,44,32,115,105,122,101,98,10,10,32,32,32,32,32,32,33,&
32,114,101,97,100,32,102,105,108,101,32,117,110,116,105,108,32,97,110,100,&
32,105,110,99,108,117,100,105,110,103,32,77,65,82,75,69,82,46,10,32,&
32,32,32,32,32,33,32,115,116,111,114,101,115,32,116,104,105,115,32,97,&
115,32,105,99,104,97,114,40,41,32,118,97,108,117,101,115,32,105,110,32,&
65,10,32,32,32,32,32,32,33,32,110,117,109,98,101,114,32,111,102,32,&
99,104,97,114,97,99,116,101,114,115,32,115,116,111,114,101,100,32,105,110,&
32,115,105,122,101,97,10,32,32,32,32,32,32,33,32,114,101,97,100,115,&
32,102,114,111,109,32,101,110,100,32,111,102,32,77,65,82,75,69,82,32,&
116,111,32,101,110,100,32,111,102,32,102,105,108,101,10,32,32,32,32,32,&
32,33,32,115,116,111,114,101,115,32,116,104,105,115,32,97,115,32,105,99,&
104,97,114,40,41,32,118,97,108,117,101,115,32,105,110,32,66,10,32,32,&
32,32,32,32,33,32,110,117,109,98,101,114,32,111,102,32,99,104,97,114,&
97,99,116,101,114,115,32,115,116,111,114,101,100,32,105,110,32,115,105,122,&
101,98,10,10,32,32,32,32,32,32,33,32,100,101,116,101,114,109,105,110&
/
data B(801:1200)/&
101,32,110,117,109,98,101,114,32,111,102,32,99,104,97,114,97,99,116,101,&
114,115,32,110,101,101,100,101,100,32,102,111,114,32,65,32,97,110,100,32,&
66,10,32,32,32,32,32,32,33,32,116,104,101,110,32,97,108,108,111,99,&
97,116,101,32,65,32,97,110,100,32,66,10,32,32,32,32,32,32,33,32,&
32,32,32,32,32,97,110,100,32,114,101,97,100,32,65,32,97,110,100,32,&
66,10,10,32,32,32,32,32,32,111,112,101,110,40,49,49,44,102,105,108,&
101,61,102,44,97,99,99,101,115,115,61,39,115,116,114,101,97,109,39,41,&
10,10,32,32,32,32,32,32,115,105,122,101,97,32,61,32,48,10,32,32,&
32,32,32,32,100,111,10,32,32,32,32,32,32,32,32,32,102,109,32,61,&
32,102,109,97,114,107,101,114,40,41,10,32,32,32,32,32,32,32,32,32,&
115,101,108,101,99,116,32,99,97,115,101,32,40,102,109,41,10,32,32,32,&
32,32,32,32,32,32,32,99,97,115,101,40,109,97,114,107,101,114,102,111,&
117,110,100,41,10,32,32,32,32,32,32,32,32,32,32,32,32,100,111,32,&
106,61,49,44,108,101,110,40,77,65,82,75,69,82,41,10,32,32,32,32,&
32,32,32,32,32,32,32,32,32,32,32,115,105,122,101,97,32,61,32,115,&
105,122,101,97,32,43,32,49,10,32,32,32,32,32,32,32,32,32,32,32,&
32,101,110,100,100,111,10,32,32,32,32,32,32,32,32,32,32,32,32,101,&
120,105,116,10,32,32,32,32,32,32,32,32,32,32,99,97,115,101,40,101,&
110,100,111,102,102,105,108,101,102,111,117,110,100,41,10,32,32,32,32,32,&
32,32,32,32,32,32,32,101,120,105,116,10,32,32,32,32,32,32,32,32&
/
data B(1201:1600)/&
32,32,99,97,115,101,32,100,101,102,97,117,108,116,10,32,32,32,32,32,&
32,32,32,32,32,32,32,115,105,122,101,97,32,61,32,115,105,122,101,97,&
32,43,32,49,10,32,32,32,32,32,32,32,32,32,101,110,100,115,101,108,&
101,99,116,10,32,32,32,32,32,32,101,110,100,100,111,10,10,32,32,32,&
32,32,32,115,105,122,101,98,32,61,32,48,10,32,32,32,32,32,32,100,&
111,10,32,32,32,32,32,32,32,32,32,114,101,97,100,32,40,49,49,44,&
101,110,100,61,49,53,48,41,32,99,10,32,32,32,32,32,32,32,32,32,&
115,105,122,101,98,32,61,32,115,105,122,101,98,32,43,32,49,10,32,32,&
32,32,32,32,101,110,100,100,111,10,49,53,48,32,32,32,99,111,110,116,&
105,110,117,101,10,10,32,32,32,32,32,32,114,101,119,105,110,100,32,49,&
49,10,10,32,32,32,32,32,32,97,108,108,111,99,97,116,101,40,65,40,&
115,105,122,101,97,41,41,10,32,32,32,32,32,32,97,108,108,111,99,97,&
116,101,40,66,40,115,105,122,101,98,41,41,10,10,32,32,32,32,32,32,&
100,111,32,105,61,49,44,115,105,122,101,97,10,32,32,32,32,32,32,32,&
32,32,114,101,97,100,40,49,49,41,99,10,32,32,32,32,32,32,32,32,&
32,65,40,105,41,32,61,32,105,110,116,40,105,99,104,97,114,40,99,41,&
44,115,104,111,114,116,105,110,116,41,10,32,32,32,32,32,32,101,110,100,&
100,111,10,10,32,32,32,32,32,32,100,111,32,105,61,49,44,115,105,122,&
101,98,10,32,32,32,32,32,32,32,32,32,114,101,97,100,40,49,49,41,&
32,99,10,32,32,32,32,32,32,32,32,32,66,40,105,41,32,61,32,105&
/
data B(1601:2000)/&
110,116,40,105,99,104,97,114,40,99,41,44,115,104,111,114,116,105,110,116,&
41,10,32,32,32,32,32,32,101,110,100,100,111,10,10,32,32,32,32,32,&
32,99,108,111,115,101,40,49,49,41,10,10,32,32,32,32,32,32,99,97,&
108,108,32,112,114,105,110,116,115,101,108,102,114,101,112,112,114,111,103,40,&
65,44,66,44,48,41,10,10,32,32,32,101,110,100,32,115,117,98,114,111,&
117,116,105,110,101,32,109,97,107,101,115,101,108,102,114,101,112,10,10,101,&
110,100,32,109,111,100,117,108,101,32,115,101,108,102,114,101,112,109,111,100,&
117,108,101,10,10,33,32,101,110,100,32,111,102,32,115,101,108,102,45,114,&
101,112,108,105,99,97,116,105,110,103,32,99,111,100,101,10,10,112,114,111,&
103,114,97,109,32,104,101,108,108,111,10,10,32,32,32,33,32,84,104,105,&
115,32,105,115,32,116,104,101,32,109,97,105,110,32,112,114,111,103,114,97,&
109,46,32,68,101,112,101,110,100,105,110,103,32,111,110,32,116,104,101,32,&
118,97,108,117,101,32,111,102,32,116,104,101,10,32,32,32,33,32,101,110,&
118,105,114,111,110,109,101,110,116,32,118,97,114,105,97,98,108,101,32,72,&
69,76,76,79,95,84,79,68,79,44,32,105,116,32,119,105,108,108,10,32,&
32,32,33,32,114,101,112,114,111,100,117,99,101,32,116,104,101,32,115,101,&
102,45,114,101,112,114,111,100,117,99,105,110,103,32,115,111,117,114,99,101,&
44,32,114,101,112,114,111,100,117,99,101,32,84,72,73,83,32,115,111,117,&
114,99,101,44,10,32,32,32,33,32,111,114,32,112,114,105,110,116,32,39,&
72,101,108,108,111,32,119,111,114,108,100,39,10,10,32,32,32,33,32,72&
/
data B(2001:2400)/&
69,76,76,79,95,84,79,68,79,61,10,32,32,32,33,32,34,34,32,111,&
114,32,97,98,115,101,110,116,58,32,114,117,110,32,105,110,32,110,111,114,&
109,97,108,32,109,111,100,101,32,40,112,114,105,110,116,32,34,72,101,108,&
108,111,32,119,111,114,108,100,34,41,10,32,32,32,33,32,34,114,101,112,&
108,105,99,97,116,101,34,32,58,32,112,114,105,110,116,32,115,101,108,102,&
32,114,101,112,114,111,100,117,99,105,110,103,32,115,111,117,114,99,101,32,&
116,111,32,115,116,100,111,117,116,10,32,32,32,33,32,34,111,114,105,103,&
105,110,97,108,34,32,58,32,112,114,105,110,116,32,84,72,73,83,32,115,&
111,117,114,99,101,32,116,111,32,115,116,100,111,117,116,10,32,32,32,33,&
32,79,116,104,101,114,119,105,122,101,32,116,104,101,32,118,97,108,117,101,&
32,105,115,32,116,97,107,101,110,32,97,115,32,97,32,102,105,108,101,32,&
110,97,109,101,46,32,84,104,105,115,32,102,105,108,101,10,32,32,32,33,&
32,119,105,108,108,32,98,101,32,114,101,97,100,32,97,110,100,32,99,111,&
110,118,101,114,116,101,100,32,116,111,32,115,101,108,102,45,114,101,112,108,&
105,99,97,116,105,110,103,32,112,114,111,103,114,97,109,46,10,32,32,32,&
33,32,32,32,32,78,111,114,109,97,108,108,121,44,32,116,104,101,32,110,&
97,109,101,32,111,102,32,84,72,73,83,32,102,105,108,101,32,119,105,108,&
108,32,98,101,32,103,105,118,101,110,46,10,10,32,32,32,117,115,101,32,&
115,101,108,102,114,101,112,109,111,100,117,108,101,10,10,32,32,32,105,109,&
112,108,105,99,105,116,32,110,111,110,101,10,10,32,32,32,99,104,97,114&
/
data B(2401:2746)/&
97,99,116,101,114,40,49,48,48,48,41,32,58,58,32,116,111,100,111,10,&
10,32,32,32,99,97,108,108,32,103,101,116,95,101,110,118,105,114,111,110,&
109,101,110,116,95,118,97,114,105,97,98,108,101,40,34,72,69,76,76,79,&
95,84,79,68,79,34,44,116,111,100,111,41,10,10,32,32,32,115,101,108,&
101,99,116,32,99,97,115,101,40,116,111,100,111,41,10,10,32,32,32,32,&
99,97,115,101,40,34,114,101,112,108,105,99,97,116,101,34,41,10,32,32,&
32,32,32,32,99,97,108,108,32,115,101,108,102,114,101,112,108,105,99,97,&
116,101,40,48,41,10,10,32,32,32,32,99,97,115,101,40,34,111,114,105,&
103,105,110,97,108,34,41,10,32,32,32,32,32,32,99,97,108,108,32,115,&
101,108,102,114,101,112,108,105,99,97,116,101,40,49,41,10,10,32,32,32,&
32,99,97,115,101,40,34,32,34,41,10,32,32,32,32,32,32,112,114,105,&
110,116,32,42,44,34,72,101,108,108,111,32,119,111,114,108,100,34,10,10,&
32,32,32,32,99,97,115,101,32,100,101,102,97,117,108,116,10,32,32,32,&
32,32,32,99,97,108,108,32,109,97,107,101,115,101,108,102,114,101,112,40,&
116,111,100,111,41,32,32,32,33,32,117,115,101,32,116,111,100,111,32,97,&
115,32,102,105,108,101,32,110,97,109,101,10,10,32,32,32,101,110,100,32,&
115,101,108,101,99,116,10,10,101,110,100,32,112,114,111,103,114,97,109,32,&
104,101,108,108,111,10/


      if (size(A) .eq. 0) then
         write(*,'(a)') 'This is not a self replicating program'
         stop
      endif

      call printselfrepprog(A,B,mode)

   end subroutine selfreplicate


   subroutine makeselfrep(f)

      implicit none
      character(*), intent(in)       :: f  ! file to read
      integer                        :: fm,j,i
      character                      :: c
      integer(shortint), allocatable :: A(:)
      integer(shortint), allocatable :: B(:)
      integer sizea, sizeb

      ! read file until and including MARKER.
      ! stores this as ichar() values in A
      ! number of characters stored in sizea
      ! reads from end of MARKER to end of file
      ! stores this as ichar() values in B
      ! number of characters stored in sizeb

      ! determine number of characters needed for A and B
      ! then allocate A and B
      !      and read A and B

      open(11,file=f,access='stream')

      sizea = 0
      do
         fm = fmarker()
         select case (fm)
          case(markerfound)
            do j=1,len(MARKER)
               sizea = sizea + 1
            enddo
            exit
          case(endoffilefound)
            exit
          case default
            sizea = sizea + 1
         endselect
      enddo

      sizeb = 0
      do
         read (11,end=150) c
         sizeb = sizeb + 1
      enddo
150   continue

      rewind 11

      allocate(A(sizea))
      allocate(B(sizeb))

      do i=1,sizea
         read(11)c
         A(i) = int(ichar(c),shortint)
      enddo

      do i=1,sizeb
         read(11) c
         B(i) = int(ichar(c),shortint)
      enddo

      close(11)

      call printselfrepprog(A,B,0)

   end subroutine makeselfrep

end module selfrepmodule

! end of self-replicating code

program hello

   ! This is the main program. Depending on the value of the
   ! environment variable HELLO_TODO, it will
   ! reproduce the sef-reproducing source, reproduce THIS source,
   ! or print 'Hello world'

   ! HELLO_TODO=
   ! "" or absent: run in normal mode (print "Hello world")
   ! "replicate" : print self reproducing source to stdout
   ! "original" : print THIS source to stdout
   ! Otherwize the value is taken as a file name. This file
   ! will be read and converted to self-replicating program.
   !    Normally, the name of THIS file will be given.

   use selfrepmodule

   implicit none

   character(1000) :: todo

   call get_environment_variable("HELLO_TODO",todo)

   select case(todo)

    case("replicate")
      call selfreplicate(0)

    case("original")
      call selfreplicate(1)

    case(" ")
      print *,"Hello world"

    case default
      call makeselfrep(todo)   ! use todo as file name

   end select

end program hello
